@@ -20,9 +20,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 package main
 
 import (
-	"os"
+	"io"
 	"path"
-	"path/filepath"
 	"strings"
 )
 
@@ -33,16 +32,16 @@ const (
 
 var TopDir string
 
-func existsDo(fdDep *os.File, cwd, pth string) (bool, error) {
+func existsDo(w io.Writer, fdDepName, cwd, pth string) (bool, error) {
 	if FileExists(path.Join(cwd, pth)) {
 		return true, nil
 	}
-	return false, ifcreate(fdDep, pth)
+	return false, ifcreate(w, fdDepName, pth)
 }
 
-func findDo(fdDep *os.File, cwd, tgt string) (string, int, error) {
+func findDo(w io.Writer, fdDepName, cwd, tgt string) (string, int, error) {
 	doFile := tgt + ".do"
-	exists, err := existsDo(fdDep, cwd, doFile)
+	exists, err := existsDo(w, fdDepName, cwd, doFile)
 	if err != nil {
 		return "", 0, err
 	}
@@ -61,7 +60,7 @@ func findDo(fdDep *os.File, cwd, tgt string) (string, int, error) {
 				[]string{"default"}, append(exts, "do")...,
 			), ".")
 			if len(levels) > 0 || (doFile != doFileOrig && doFile != tgt) {
-				exists, err = existsDo(fdDep, cwd, path.Join(updir, doFile))
+				exists, err = existsDo(w, fdDepName, cwd, path.Join(updir, doFile))
 				if err != nil {
 					return "", 0, err
 				}
@@ -73,7 +72,7 @@ func findDo(fdDep *os.File, cwd, tgt string) (string, int, error) {
 		}
 		doFile = "default.do"
 		if len(levels) > 0 || (doFile != doFileOrig && doFile != tgt) {
-			exists, err = existsDo(fdDep, cwd, path.Join(updir, doFile))
+			exists, err = existsDo(w, fdDepName, cwd, path.Join(updir, doFile))
 			if err != nil {
 				return "", 0, err
 			}
@@ -82,7 +81,7 @@ func findDo(fdDep *os.File, cwd, tgt string) (string, int, error) {
 			}
 		}
 		levels = append(levels, "..")
-		dirAbs, err := filepath.Abs(path.Join(cwd, updir))
+		dirAbs := mustAbs(path.Join(cwd, updir))
 		if err != nil {
 			panic(err)
 		}
