@@ -24,7 +24,7 @@ import (
 )
 
 const (
-	Version  = "1.32.0"
+	Version  = "2.0.0"
 	Warranty = `Copyright (C) 2020-2023 Sergey Matveev
 
 This program is free software: you can redistribute it and/or modify
@@ -105,18 +105,28 @@ List all currently known out-of-date targets.`
 		d = `Usage: redo-affects target [...]
 
 List all targets that will be affected by changing the specified ones.`
+	case CmdNameRedoDep2Rec:
+		d = `Usage: redo-dep2rec .../.redo/file.dep
+
+Convert binary .dep file to recfile and write it to stdout.`
 	case CmdNameRedoDepFix:
 		d = `Usage: redo-depfix
 
-Traverse over all .redo directories beneath and check if inode's information
-(ctime/mtime) differs. Update dependency if file's content is still the same.`
+Traverse over all .redo directories beneath and recalculate each target's
+inode and hash information, rewriting dependency files. If dependency has
+legacy .rec format, then it will be converted to .dep one.`
+	case CmdNameRedoInode:
+		d = `Usage: redo-inode target [...]
+
+Just calculate inode information about each target and print in recfile format.`
 	default:
 		d = `Usage: goredo -symlinks
 
 goredo expects to be called through the symbolic link to it.
 Available commands: redo, redo-affects, redo-always, redo-cleanup,
-redo-depfix, redo-dot, redo-ifchange, redo-ifcreate, redo-log,
-redo-ood, redo-sources, redo-stamp, redo-targets, redo-whichdo.`
+redo-dep2rec, redo-depfix, redo-dot, redo-ifchange, redo-ifcreate,
+redo-inode, redo-log, redo-ood, redo-sources, redo-stamp,
+redo-targets, redo-whichdo.`
 	}
 	fmt.Fprintf(os.Stderr, "%s\n\nCommon options:\n", d)
 	flag.PrintDefaults()
